import os
from glob import glob
from os.path import join, basename

from ..util import replace_ext, pairwise, gdal, osr


def browse_georeference(source_dir: os.PathLike, target_dir: os.PathLike, browse: dict):
    for filename in glob(join(source_dir, '*')):
        target_filename = join(target_dir, replace_ext(basename(filename), '.tif'))
        apply_browse_report_georeference(filename, target_filename, browse)


def apply_browse_report_georeference(input_filename: os.PathLike, target_filename: os.PathLike, browse: dict):
    ds = gdal.GetDriverByName('GTiff').CreateCopy(target_filename, gdal.Open(input_filename))
    type_ = browse['type']

    if type_ == 'rectified_browse':
        size_x, size_y = ds.RasterXSize, ds.RasterYSize
        low, high = browse['rectified']['coord_list']

        minx, miny = low
        maxx, maxy = high

        ds.SetGeoTransform([
            minx, (maxx - minx) / size_x, 0,
            maxy, 0, (miny - maxy) / size_y,
        ])

    elif type_ == 'footprint_browse':
        col_rows = browse['footprint']['col_row_list']
        coords = browse['footprint']['coord_list']

        if col_rows[0] == col_rows[-1] and coords[0] == coords[-1]:
            col_rows = col_rows[:-1]
            coords = coords[:-1]

        gcps = [
            gdal.GCP(coord[1], coord[0], 0, col_row[0], col_row[1])
            for col_row, coord in zip(col_rows, coords)
        ]

        sr = osr.SpatialReference()
        sr.ImportFromEPSG(4326)
        ds.SetGCPs(gcps, sr)

    elif type_ == 'model_in_geotiff_browse':
        # nothing to do in this case
        pass

    elif type_ == 'regular_grid_browse':
        col_node_number = browse['regular_grid']['col_node_number']
        row_node_number = browse['regular_grid']['row_node_number']
        col_step = browse['regular_grid']['col_step']
        row_step = browse['regular_grid']['row_step']

        coord_lists = browse['regular_grid']['coord_lists']

        range_x = frange(0.0, row_node_number * row_step, row_step)
        range_y = frange(0.0, col_node_number * col_step, col_step)
        pixels = [(x, y) for x in range_x for y in range_y]

        coords = [
            coord
            for coord in coord_list for coord_list in coord_lists
        ]

        gcps = [
            gdal.GCP(coord[1], coord[0], 0, col_row[0], col_row[1])
            for col_row, coord in zip(col_rows, coords)
        ]

        sr = osr.SpatialReference()
        sr.ImportFromEPSG(4326)
        ds.SetGCPs(gcps, sr)

    else:
        raise Exception

    del ds


# copied from: https://pynative.com/python-range-for-float-numbers/
def frange(start, stop=None, step=None):
    # if stop and step argument is None set start=0.0 and step = 1.0
    start = float(start)
    if stop == None:
        stop = start + 0.0
        start = 0.0
    if step == None:
        step = 1.0

    count = 0
    while True:
        temp = float(start + count * step)
        if step > 0 and temp >= stop:
            break
        elif step < 0 and temp <= stop:
            break
        yield temp

        count += 1
